"""This module contains the TextElement class for the text element model."""
from t_page_object.base_element import BaseElement


class TextElement(BaseElement):
    """Class for input element model."""

    pass
