#!/usr/bin/env python
"""Tests for `t_page_object` package."""


class TestTPageObject:
    """Smoke tests of the package."""

    def test_example(self):
        """Smoke test."""
        assert True
